
#include <winsock.h>
#include <io.h>
#include <stdio.h>
#pragma warning(disable:4270)

#include <windows.h>

#define dll_long __declspec(dllexport) long __stdcall
#define dll_void __declspec(dllexport) void __stdcall

#include <openssl/crypto.h>
#include <openssl/x509.h>
#include <openssl/pem.h>
#include <openssl/ssl.h>
#include <openssl/err.h>
#include <openssl/rand.h>

BOOL APIENTRY DllMain(HANDLE hModule, DWORD ul_reason_for_call, LPVOID lpReserved)
{                           
   return TRUE;
}

struct SSLDATA
{
   SSL_METHOD* meth;
   SSL_CTX* ctx;
   SSL* ssl;
};

static RSA *tmp_rsa_cb(SSL *s, int is_export, int keylength)
{
   static RSA *rsa_tmp=NULL;
   if (!rsa_tmp) rsa_tmp=RSA_generate_key(keylength,RSA_F4,NULL,NULL);
   return(rsa_tmp);
}

dll_long ssl_init_client(long socket, char *cafile)
{
   CRYPTO_malloc_init();
   SSLDATA *ssldata=new SSLDATA;

   RAND_screen();
   SSL_load_error_strings();
   OpenSSL_add_ssl_algorithms();
   
   ssldata->meth=SSLv2_client_method();
   ssldata->ctx=SSL_CTX_new(ssldata->meth); if (!ssldata->ctx) { delete ssldata; return 0; }

   ssldata->ssl=SSL_new(ssldata->ctx); if (!ssldata->ssl) { delete ssldata; return 0; }
   SSL_set_fd(ssldata->ssl, (int)socket);

   if (cafile && *cafile)
   {
      SSL_CTX_set_verify(ssldata->ctx,SSL_VERIFY_PEER, SSL_get_verify_callback(ssldata->ssl)); // SSL_VERIFY_NONE
	   SSL_CTX_load_verify_locations(ssldata->ctx, cafile, NULL);
   }

   if (SSL_connect(ssldata->ssl)==-1) { delete ssldata; return 0; }

   return (long)ssldata;
}

dll_long ssl_init_server(long socket, char *cert, char *key)
{
   CRYPTO_malloc_init();
   SSLDATA *ssldata=new SSLDATA;

   RAND_screen();
   SSL_load_error_strings();
   OpenSSL_add_ssl_algorithms();

   ssldata->meth=SSLv23_server_method();
   ssldata->ctx=SSL_CTX_new(ssldata->meth); if (!ssldata->ctx) { delete ssldata; return 0; }
	SSL_CTX_sess_set_cache_size(ssldata->ctx,128);

   if (SSL_CTX_use_certificate_file(ssldata->ctx, cert, SSL_FILETYPE_PEM)<=0) { delete ssldata; return 0; }
   if (SSL_CTX_use_PrivateKey_file(ssldata->ctx, key, SSL_FILETYPE_PEM)<=0) { delete ssldata; return 0; }
   if (!SSL_CTX_check_private_key(ssldata->ctx)) { delete ssldata; return 0; }

	SSL_CTX_set_tmp_rsa_callback(ssldata->ctx,tmp_rsa_cb);

   ssldata->ssl=SSL_new(ssldata->ctx); if (!ssldata->ssl) { delete ssldata; return 0; }
   SSL_set_fd(ssldata->ssl, (int)socket);
   if (SSL_accept(ssldata->ssl)==-1) { delete ssldata; return 0; } 

   return (long)ssldata;
}

dll_void ssl_getcipher(long handle, char *puffer, int length)
{
   if (!handle) return;
   SSLDATA *ssldata=(SSLDATA *)handle;
   _snprintf(puffer, length, "%s", SSL_get_cipher(ssldata->ssl));
}

dll_void ssl_getsubject(long handle, char *puffer, int length)
{
   if (!handle) return;
   SSLDATA *ssldata=(SSLDATA *)handle;
   X509* cert=SSL_get_peer_certificate(ssldata->ssl);
   
   if (cert)
   {
      char *subject=X509_NAME_oneline(X509_get_subject_name(cert), 0, 0);
      _snprintf(puffer, length, "%s", subject);
      if (subject) Free(subject);
   }
   else
      _snprintf(puffer, length, "no subject");

   X509_free(cert);
}

dll_void ssl_getissuer(long handle, char *puffer, int length)
{
   if (!handle) return;
   SSLDATA *ssldata=(SSLDATA *)handle;
   X509* cert=SSL_get_peer_certificate(ssldata->ssl);

   if (cert)
   {
      char *issuer=X509_NAME_oneline(X509_get_issuer_name(cert),0,0);
      _snprintf(puffer, length, "%s", issuer);
      if (issuer) Free(issuer);
   }
   else
      _snprintf(puffer, length, "%s", "no subject");

   X509_free(cert);
}

dll_long ssl_send(long handle, char *data, long length)
{
   if (!handle) return 0;
   SSLDATA *ssldata=(SSLDATA *)handle;
   return SSL_write(ssldata->ssl, data, length);
}

dll_long ssl_recv(long handle, char *data, long length)
{
   if (!handle) return 0;
   SSLDATA *ssldata=(SSLDATA *)handle;
   return SSL_read(ssldata->ssl, data, length);
}

dll_void ssl_cleanup(long handle)
{
   if (!handle) return;
   SSLDATA *ssldata=(SSLDATA *)handle;
   SSL_shutdown(ssldata->ssl);
   SSL_free(ssldata->ssl);
   SSL_CTX_free(ssldata->ctx);
   delete ssldata;
}

dll_long ssl_getverify(long handle)
{
   if (!handle) return 0;
   SSLDATA *ssldata=(SSLDATA *)handle;
   return SSL_get_verify_result(ssldata->ssl);
}
